function v = get(d,p)
%GET    Get database metadata properties.
%   V = GET(D,'PropertyName') returns the value of the specified 
%   properties for the database meta data object, D.  'PropertyName' is
%   a string or cell array of strings containing property names.
%
%   V = GET(D) returns a structure where each field name is the name
%   of a property of D and each field contains the value of that 
%   property.
%
%   See also SUPPORTS.

%   Author(s): C.F.Garvin, 07-09-98
%   Copyright 1984-2002 The MathWorks, Inc.
%   $Revision: 1.11 $   $Date: 2002/06/17 12:01:21 $

%Build properties if none are given   
prps = {'AllProceduresAreCallable';...
        'AllTablesAreSelectable';...
        'DataDefinitionCausesTransactionCommit';...
        'DataDefinitionIgnoredInTransactions';...
        'DoesMaxRowSizeIncludeBlobs';...
        'Catalogs';...
        'CatalogSeparator';...
        'CatalogTerm';...
        'DatabaseProductName';...
        'DatabaseProductVersion';...
        'DefaultTransactionIsolation';...
        'DriverMajorVersion';...
        'DriverMinorVersion';...
        'DriverName';...
        'DriverVersion';...
        'ExtraNameCharacters';...
        'IdentifierQuoteString';...
        'IsCatalogAtStart';...
        'MaxBinaryLiteralLength';...
        'MaxCatalogNameLength';...
        'MaxCharLiteralLength';...
        'MaxColumnNameLength';...
        'MaxColumnsInGroupBy';...
        'MaxColumnsInIndex';...
        'MaxColumnsInOrderBy';...
        'MaxColumnsInSelect';...
        'MaxColumnsInTable';...
        'MaxConnections';...
        'MaxCursorNameLength';...
        'MaxIndexLength';...
        'MaxProcedureNameLength';...
        'MaxRowSize';...
        'MaxSchemaNameLength';...
        'MaxStatementLength';...
        'MaxStatements';...
        'MaxTableNameLength';...
        'MaxTablesInSelect';...
        'MaxUserNameLength';...
        'NumericFunctions';...
        'ProcedureTerm';...
        'Schemas';...
        'SchemaTerm';...
        'SearchStringEscape';...
        'SQLKeywords';...
        'StringFunctions';...
        'StoresLowerCaseIdentifiers';...
        'StoresLowerCaseQuotedIdentifiers';...
        'StoresMixedCaseIdentifiers';...
        'StoresMixedCaseQuotedIdentifiers';...
        'StoresUpperCaseIdentifiers';...
        'StoresUpperCaseQuotedIdentifiers';...
        'SystemFunctions';...
        'TableTypes';...
        'TimeDateFunctions';...
        'TypeInfo';...
        'URL';...
        'UserName';...
        'NullPlusNonNullIsNull';...
        'NullsAreSortedAtEnd';...
        'NullsAreSortedAtStart';...
        'NullsAreSortedHigh';...
        'NullsAreSortedLow';...
        'UsesLocalFilePerTable';...
        'UsesLocalFiles';...
       };   

%Determine which properties are requested
if nargin == 1
  p = prps;
else
  p = chkprops(d,p,prps);
end

%DatabaseDMD instance
a = com.mathworks.toolbox.database.databaseDMD;

%Get property values, use javaMethod as some method names > 33 characters
for i = 1:length(p)
  eval(['v.' p{i} ' = javaMethod(''dmd' p{i} ''',a,d.DMDHandle);'])
  if any(strcmp(p{i},{'Catalogs','Schemas','TableTypes','TypeInfo'}))
    eval(['v.' p{i} ' = { ' eval(['v.' p{i}]) '};'])
  end
end

if length(p) == 1
  eval(['v = v.' char(p) ';'])
end
