function v = supports(d,p)
%SUPPORTS Detect if property is supported by database metadata object.
%   V = SUPPORTS(D,'PropertyName') returns 1 if the specified 
%   properties for the database meta data object, D, are supported and
%   0 otherwise.  'PropertyName' is a string or cell array of strings 
%   containing property names.
%
%   V = SUPPORTS(D) returns a structure where each field name is the 
%   name of a property of D and each field contains a 1 or 0 for a
%   supported or unsupported property, respectively.
%
%   See also GET.

%   Author(s): C.F.Garvin, 07-09-98
%   Copyright 1984-2002 The MathWorks, Inc.
%   $Revision: 1.10 $   $Date: 2002/06/17 12:01:23 $

%Build properties if none are given   
prps = {'AlterTableWithAddColumn';...
    'AlterTableWithDropColumn';...
    'ANSI92EntryLevelSQL';...
    'ANSI92FullSQL';...
    'ANSI92IntermediateSQL';...
    'CatalogsInDataManipulation';...
    'CatalogsInIndexDefinitions';...
    'CatalogsInPrivilegeDefinitions';...
    'CatalogsInProcedureCalls';...
    'CatalogsInTableDefinitions';...
    'ColumnAliasing';...
    'Convert';...
    'CoreSQLGrammar';...
    'CorrelatedSubqueries';...
    'DataDefinitionAndDataManipulationTransactions';...
    'DataManipulationTransactionsOnly';...
    'DifferentTableCorrelationNames';...
    'ExpressionsInOrderBy';...
    'ExtendedSQLGrammar';...
    'FullOuterJoins';...
    'GroupBy';...
    'GroupByBeyondSelect';...
    'GroupByUnrelated';...
    'IntegrityEnhancementFacility';...
    'LikeEscapeClause';...
    'LimitedOuterJoins';...
    'MinimumSQLGrammar';...
    'MixedCaseIdentifiers';...
    'MixedCaseQuotedIdentifiers';...
    'MultipleResultSets';...
    'MultipleTransactions';...
    'NonNullableColumns';...
    'OpenCursorsAcrossCommit';...
    'OpenCursorsAcrossRollback';...
    'OpenStatementsAcrossCommit';...
    'OpenStatementsAcrossRollback';...
    'OrderByUnrelated';...
    'OuterJoins';...
    'PositionedDelete';...
    'PositionedUpdate';...
    'SchemasInDataManipulation';...
    'SchemasInIndexDefinitions';...
    'SchemasInPrivilegeDefinitions';...
    'SchemasInProcedureCalls';...
    'SchemasInTableDefinitions';...
    'SelectForUpdate';...
    'StoredProcedures';...
    'SubqueriesInComparisons';...
    'SubqueriesInExists';...
    'SubqueriesInIns';...
    'SubqueriesInQuantifieds';...
    'TableCorrelationNames';...
    'Transactions';...
    'Union';...
    'UnionAll';...
    };   

%Determine which properties are requested
if nargin == 1
  p = prps;
else
  p= chkprops(d,p,prps);
end

%Get property values, use javaMethod as some method names > 33 characters
for i = 1:length(p)
  eval(['v.' p{i} ' = javaMethod(''supports' p{i} ''',d.DMDHandle);'])
end

if length(p) == 1
  eval(['v = v.' char(p) ';'])
end
